import { __decorate } from 'tslib';
import { ViewChild, Input, Component, forwardRef, ViewEncapsulation, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Plugin, PluginKey, EditorState } from 'prosemirror-state';
import { DecorationSet, Decoration, EditorView } from 'prosemirror-view';
import { schema } from 'prosemirror-schema-basic';
import { undo, redo, history } from 'prosemirror-history';
import { keymap } from 'prosemirror-keymap';
import { toggleMark, baseKeymap } from 'prosemirror-commands';
import classNames from 'classnames';

const MENU_ITEM_CLASSNAME = 'NgxEditor-MenuItem';
const MENU_WRAPPER_CLASSNAME = 'NgxEditor-MenuBar';

/* tslint:disable */
var bold = `
  <path d="M15.6 10.79c.97-.67 1.65-1.77 1.65-2.79 0-2.26-1.75-4-4-4H7v14h7.04c2.09 0 3.71-1.7 3.71-3.79 0-1.52-.86-2.82-2.15-3.42zM10 6.5h3c.83 0 1.5.67 1.5 1.5s-.67 1.5-1.5 1.5h-3v-3zm3.5 9H10v-3h3.5c.83 0 1.5.67 1.5 1.5s-.67 1.5-1.5 1.5z" />
  <path d="M0 0h24v24H0z" fill="none" />
`;

var italic = `
  <path d="M0 0h24v24H0z" fill="none" />
  <path d="M10 4v3h2.21l-3.42 8H6v3h8v-3h-2.21l3.42-8H18V4z" />
`;

var code = `
<path d="M0 0h24v24H0V0z" fill="none"/>
<path d="M9.4 16.6L4.8 12l4.6-4.6L8 6l-6 6 6 6 1.4-1.4zm5.2 0l4.6-4.6-4.6-4.6L16 6l6 6-6 6-1.4-1.4z"/>
`;

// Icons source: https://material.io/
const height = 20;
const width = 20;
const icons = {
    bold,
    italic,
    code
};
// Helper function to create menu icons
function getIconSvg(name) {
    const path = icons[name] || '<path></path>';
    return `
  <svg
    xmlns="http://www.w3.org/2000/svg"
    viewBox="0 0 24 24"
    fill="black"
    height=${height}
    width=${width}
  >
    ${path}
  </svg>
  `;
}

function getMenuDom(name) {
    const div = document.createElement('div');
    div.className = classNames(MENU_ITEM_CLASSNAME, `${MENU_ITEM_CLASSNAME}__${name}`);
    div.title = name;
    div.innerHTML = getIconSvg(name);
    return div;
}
function getSeperatorDom() {
    const div = document.createElement('div');
    div.className = classNames(`${MENU_ITEM_CLASSNAME}__Seperator`);
    return div;
}
const menuItems = {
    bold: { key: 'strong', command: toggleMark(schema.marks.strong), dom: getMenuDom('bold') },
    italic: { key: 'em', command: toggleMark(schema.marks.em), dom: getMenuDom('italic') },
    code: { key: 'code', command: toggleMark(schema.marks.code), dom: getMenuDom('code') }
};
const seperator = { key: 'seperator', dom: getSeperatorDom() };
const getMenu = (toolbar) => {
    const menu = [];
    toolbar.forEach((group, toolbarIndex) => {
        const isLastMenuGroup = toolbar.length - 1 === toolbarIndex;
        group.forEach((menuName, menuIndex) => {
            const isLastMenuItem = group.length - 1 === menuIndex;
            const menuItem = menuItems[menuName];
            if (menuItem) {
                menu.push(menuItem);
                if (isLastMenuItem && !isLastMenuGroup) {
                    menu.push(seperator);
                }
            }
        });
    });
    return menu;
};
const ɵ0 = getMenu;

class MenuBarView {
    constructor(toolbar, editorView) {
        const menu = getMenu(toolbar);
        this.editorView = editorView;
        // remove elements without commands
        this.menu = menu.filter(menuItem => menuItem.command);
        this.dom = document.createElement('div');
        this.dom.className = MENU_WRAPPER_CLASSNAME;
        menu.forEach(({ dom }) => this.dom.appendChild(dom));
        this.update();
        this.dom.addEventListener('mousedown', (e) => {
            e.preventDefault();
            editorView.focus();
            this.menu.forEach(({ command, dom }) => {
                if (dom.contains(e.target)) {
                    command(editorView.state, editorView.dispatch, editorView);
                }
            });
        });
    }
    getIsElementActive(state, type) {
        const { from, $from, to, empty } = state.selection;
        if (empty) {
            return type.isInSet(state.storedMarks || $from.marks());
        }
        else {
            return state.doc.rangeHasMark(from, to, type);
        }
    }
    update() {
        this.menu.forEach(({ command, dom, key }) => {
            const canShowItem = command(this.editorView.state, null, this.editorView);
            const state = this.editorView.state;
            const markType = state.schema.marks[key];
            const isActive = !!this.getIsElementActive(state, markType);
            dom.style.display = canShowItem ? '' : 'none';
            dom.classList.toggle(`${MENU_ITEM_CLASSNAME}__active`, isActive);
        });
    }
    destroy() {
        this.dom.remove();
    }
}

function menuPlugin(toolbar) {
    return new Plugin({
        key: new PluginKey('menu'),
        view(editorView) {
            const menuView = new MenuBarView(toolbar, editorView);
            editorView.dom.parentNode.insertBefore(menuView.dom, editorView.dom);
            return menuView;
        },
    });
}
const menu = (toolbarOptions) => {
    return menuPlugin(toolbarOptions);
};
const ɵ0$1 = menu;

function placeholderPlugin(text) {
    return new Plugin({
        key: new PluginKey('placeholder'),
        props: {
            decorations(state) {
                const doc = state.doc;
                if (doc.childCount === 1 && doc.firstChild.isTextblock && doc.firstChild.content.size === 0) {
                    const placeHolderEl = document.createElement('span');
                    placeHolderEl.classList.add('NgxEditor-Placeholder');
                    placeHolderEl.textContent = text;
                    return DecorationSet.create(doc, [Decoration.widget(1, placeHolderEl)]);
                }
                return DecorationSet.empty;
            }
        }
    });
}

const isMacOs = /Mac/.test(navigator.platform);
function getHistoryKeyMap() {
    const historyMap = {};
    historyMap['Mod-z'] = undo;
    if (isMacOs) {
        historyMap['Shift-Mod-z'] = redo;
    }
    else {
        historyMap['Mod-y'] = redo;
    }
    return historyMap;
}
const getPlugins = (options) => {
    const historyKeyMap = getHistoryKeyMap();
    const plugins = [
        history(),
        keymap(baseKeymap),
        keymap(historyKeyMap),
        keymap({
            'Mod-b': toggleMark(schema.marks.strong),
            'Mod-i': toggleMark(schema.marks.em),
            'Mod-`': toggleMark(schema.marks.code)
        }),
        placeholderPlugin(options.placeholder),
    ];
    if (options.toolbar) {
        plugins.push(menu(options.toolbar));
    }
    return plugins;
};

const defaultToolbarOptions = [
    ['bold', 'italic', 'code'],
];
const defaultConfig = {
    toolbar: true
};
const getToolbar = (config) => {
    let toolbar = null;
    // set toolbar options
    if (typeof config.toolbar === 'boolean') {
        if (config.toolbar) {
            toolbar = defaultToolbarOptions;
        }
    }
    else {
        toolbar = config.toolbar;
    }
    return toolbar;
};
const ɵ0$2 = getToolbar;
// prefer attributes than config
const computeOptions = (options) => {
    const computedOptions = {};
    const config = Object.assign({}, options.config, defaultConfig);
    computedOptions.toolbar = getToolbar(config);
    if (options.placeholder) {
        computedOptions.placeholder = options.placeholder;
    }
    return computedOptions;
};
const ɵ1 = computeOptions;

var NgxEditorComponent_1;
let NgxEditorComponent = NgxEditorComponent_1 = class NgxEditorComponent {
    constructor() {
        this.placeholder = 'Type here...';
    }
    writeValue(value) {
        if (!value) {
            return;
        }
        this.updateContent(value);
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched() { }
    parseDoc(contentJson) {
        return schema.nodeFromJSON(contentJson);
    }
    updateContent(value) {
        try {
            const doc = this.parseDoc(value);
            const state = this.view.state;
            const tr = state.tr;
            tr.replaceWith(0, state.doc.content.size, doc);
            this.view.dispatch(tr);
        }
        catch (err) {
            console.error('Unable to update document.', err);
        }
    }
    handleTransactions(tr) {
        const { state } = this.view.state.applyTransaction(tr);
        this.view.updateState(state);
        if (tr.docChanged) {
            const json = state.doc.toJSON();
            this.onChange(json);
        }
    }
    createEditor() {
        this.view = new EditorView(this.ngxEditor.nativeElement, {
            state: EditorState.create({
                schema,
                plugins: getPlugins(this.options),
            }),
            dispatchTransaction: this.handleTransactions.bind(this),
            attributes: {
                class: 'NgxEditor-Content'
            },
        });
    }
    ngOnInit() {
        this.options = computeOptions({
            placeholder: this.placeholder,
            config: this.config
        });
        this.createEditor();
    }
    ngOnDestroy() {
        this.view.destroy();
    }
};
__decorate([
    ViewChild('ngxEditor', { static: true })
], NgxEditorComponent.prototype, "ngxEditor", void 0);
__decorate([
    Input()
], NgxEditorComponent.prototype, "placeholder", void 0);
__decorate([
    Input()
], NgxEditorComponent.prototype, "config", void 0);
NgxEditorComponent = NgxEditorComponent_1 = __decorate([
    Component({
        selector: 'ngx-editor',
        template: "<div class=\"NgxEditor-Wrapper\">\n  <div #ngxEditor></div>\n</div>\n",
        providers: [{
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => NgxEditorComponent_1),
                multi: true
            }],
        encapsulation: ViewEncapsulation.None,
        styles: [".ProseMirror{outline:0}.NgxEditor-Wrapper{background:padding-box #fff;color:#000;border-radius:4px;border:2px solid rgba(0,0,0,.2)}.NgxEditor-MenuBar{display:flex;padding:.2rem;border-bottom:1px solid #ddd}.NgxEditor-MenuBar .NgxEditor-MenuItem{height:30px;width:30px;display:flex;align-items:center;justify-content:center;transition:.3s ease-in-out;border-radius:4px}.NgxEditor-MenuBar .NgxEditor-MenuItem:hover{cursor:pointer;background-color:#f1f1f1}.NgxEditor-MenuBar .NgxEditor-MenuItem.NgxEditor-MenuItem__active svg{fill:#06c}.NgxEditor-MenuBar .NgxEditor-MenuItem__Seperator{border-left:1px solid #ccc;margin:0 5px}.NgxEditor-Content{padding:.5rem}.NgxEditor-Content p{margin:0 0 .7rem}.NgxEditor-Placeholder{color:#6c757d;opacity:1}"]
    })
], NgxEditorComponent);

let NgxEditorModule = class NgxEditorModule {
};
NgxEditorModule = __decorate([
    NgModule({
        declarations: [NgxEditorComponent],
        exports: [NgxEditorComponent],
    })
], NgxEditorModule);

/**
 * Generated bundle index. Do not edit.
 */

export { NgxEditorModule, NgxEditorComponent as ɵa };
//# sourceMappingURL=ngx-editor.js.map
