import { __decorate } from 'tslib';
import { ViewChild, Input, Component, forwardRef, ViewEncapsulation, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Plugin, PluginKey, EditorState } from 'prosemirror-state';
import { DecorationSet, Decoration, EditorView } from 'prosemirror-view';
import { schema } from 'prosemirror-schema-basic';
import { undo, redo, history } from 'prosemirror-history';
import { keymap } from 'prosemirror-keymap';
import { toggleMark, baseKeymap } from 'prosemirror-commands';
import classNames from 'classnames';

var MENU_ITEM_CLASSNAME = 'NgxEditor-MenuItem';
var MENU_WRAPPER_CLASSNAME = 'NgxEditor-MenuBar';

/* tslint:disable */
var bold = "\n  <path d=\"M15.6 10.79c.97-.67 1.65-1.77 1.65-2.79 0-2.26-1.75-4-4-4H7v14h7.04c2.09 0 3.71-1.7 3.71-3.79 0-1.52-.86-2.82-2.15-3.42zM10 6.5h3c.83 0 1.5.67 1.5 1.5s-.67 1.5-1.5 1.5h-3v-3zm3.5 9H10v-3h3.5c.83 0 1.5.67 1.5 1.5s-.67 1.5-1.5 1.5z\" />\n  <path d=\"M0 0h24v24H0z\" fill=\"none\" />\n";

var italic = "\n  <path d=\"M0 0h24v24H0z\" fill=\"none\" />\n  <path d=\"M10 4v3h2.21l-3.42 8H6v3h8v-3h-2.21l3.42-8H18V4z\" />\n";

var code = "\n<path d=\"M0 0h24v24H0V0z\" fill=\"none\"/>\n<path d=\"M9.4 16.6L4.8 12l4.6-4.6L8 6l-6 6 6 6 1.4-1.4zm5.2 0l4.6-4.6-4.6-4.6L16 6l6 6-6 6-1.4-1.4z\"/>\n";

// Icons source: https://material.io/
var height = 20;
var width = 20;
var icons = {
    bold: bold,
    italic: italic,
    code: code
};
// Helper function to create menu icons
function getIconSvg(name) {
    var path = icons[name] || '<path></path>';
    return "\n  <svg\n    xmlns=\"http://www.w3.org/2000/svg\"\n    viewBox=\"0 0 24 24\"\n    fill=\"black\"\n    height=" + height + "\n    width=" + width + "\n  >\n    " + path + "\n  </svg>\n  ";
}

function getMenuDom(name) {
    var div = document.createElement('div');
    div.className = classNames(MENU_ITEM_CLASSNAME, MENU_ITEM_CLASSNAME + "__" + name);
    div.title = name;
    div.innerHTML = getIconSvg(name);
    return div;
}
function getSeperatorDom() {
    var div = document.createElement('div');
    div.className = classNames(MENU_ITEM_CLASSNAME + "__Seperator");
    return div;
}
var menuItems = {
    bold: { key: 'strong', command: toggleMark(schema.marks.strong), dom: getMenuDom('bold') },
    italic: { key: 'em', command: toggleMark(schema.marks.em), dom: getMenuDom('italic') },
    code: { key: 'code', command: toggleMark(schema.marks.code), dom: getMenuDom('code') }
};
var seperator = { key: 'seperator', dom: getSeperatorDom() };
var getMenu = function (toolbar) {
    var menu = [];
    toolbar.forEach(function (group, toolbarIndex) {
        var isLastMenuGroup = toolbar.length - 1 === toolbarIndex;
        group.forEach(function (menuName, menuIndex) {
            var isLastMenuItem = group.length - 1 === menuIndex;
            var menuItem = menuItems[menuName];
            if (menuItem) {
                menu.push(menuItem);
                if (isLastMenuItem && !isLastMenuGroup) {
                    menu.push(seperator);
                }
            }
        });
    });
    return menu;
};
var ɵ0 = getMenu;

var MenuBarView = /** @class */ (function () {
    function MenuBarView(toolbar, editorView) {
        var _this = this;
        var menu = getMenu(toolbar);
        this.editorView = editorView;
        // remove elements without commands
        this.menu = menu.filter(function (menuItem) { return menuItem.command; });
        this.dom = document.createElement('div');
        this.dom.className = MENU_WRAPPER_CLASSNAME;
        menu.forEach(function (_a) {
            var dom = _a.dom;
            return _this.dom.appendChild(dom);
        });
        this.update();
        this.dom.addEventListener('mousedown', function (e) {
            e.preventDefault();
            editorView.focus();
            _this.menu.forEach(function (_a) {
                var command = _a.command, dom = _a.dom;
                if (dom.contains(e.target)) {
                    command(editorView.state, editorView.dispatch, editorView);
                }
            });
        });
    }
    MenuBarView.prototype.getIsElementActive = function (state, type) {
        var _a = state.selection, from = _a.from, $from = _a.$from, to = _a.to, empty = _a.empty;
        if (empty) {
            return type.isInSet(state.storedMarks || $from.marks());
        }
        else {
            return state.doc.rangeHasMark(from, to, type);
        }
    };
    MenuBarView.prototype.update = function () {
        var _this = this;
        this.menu.forEach(function (_a) {
            var command = _a.command, dom = _a.dom, key = _a.key;
            var canShowItem = command(_this.editorView.state, null, _this.editorView);
            var state = _this.editorView.state;
            var markType = state.schema.marks[key];
            var isActive = !!_this.getIsElementActive(state, markType);
            dom.style.display = canShowItem ? '' : 'none';
            dom.classList.toggle(MENU_ITEM_CLASSNAME + "__active", isActive);
        });
    };
    MenuBarView.prototype.destroy = function () {
        this.dom.remove();
    };
    return MenuBarView;
}());

function menuPlugin(toolbar) {
    return new Plugin({
        key: new PluginKey('menu'),
        view: function (editorView) {
            var menuView = new MenuBarView(toolbar, editorView);
            editorView.dom.parentNode.insertBefore(menuView.dom, editorView.dom);
            return menuView;
        },
    });
}
var menu = function (toolbarOptions) {
    return menuPlugin(toolbarOptions);
};
var ɵ0$1 = menu;

function placeholderPlugin(text) {
    return new Plugin({
        key: new PluginKey('placeholder'),
        props: {
            decorations: function (state) {
                var doc = state.doc;
                if (doc.childCount === 1 && doc.firstChild.isTextblock && doc.firstChild.content.size === 0) {
                    var placeHolderEl = document.createElement('span');
                    placeHolderEl.classList.add('NgxEditor-Placeholder');
                    placeHolderEl.textContent = text;
                    return DecorationSet.create(doc, [Decoration.widget(1, placeHolderEl)]);
                }
                return DecorationSet.empty;
            }
        }
    });
}

var isMacOs = /Mac/.test(navigator.platform);
function getHistoryKeyMap() {
    var historyMap = {};
    historyMap['Mod-z'] = undo;
    if (isMacOs) {
        historyMap['Shift-Mod-z'] = redo;
    }
    else {
        historyMap['Mod-y'] = redo;
    }
    return historyMap;
}
var getPlugins = function (options) {
    var historyKeyMap = getHistoryKeyMap();
    var plugins = [
        history(),
        keymap(baseKeymap),
        keymap(historyKeyMap),
        keymap({
            'Mod-b': toggleMark(schema.marks.strong),
            'Mod-i': toggleMark(schema.marks.em),
            'Mod-`': toggleMark(schema.marks.code)
        }),
        placeholderPlugin(options.placeholder),
    ];
    if (options.toolbar) {
        plugins.push(menu(options.toolbar));
    }
    return plugins;
};

var defaultToolbarOptions = [
    ['bold', 'italic', 'code'],
];
var defaultConfig = {
    toolbar: true
};
var getToolbar = function (config) {
    var toolbar = null;
    // set toolbar options
    if (typeof config.toolbar === 'boolean') {
        if (config.toolbar) {
            toolbar = defaultToolbarOptions;
        }
    }
    else {
        toolbar = config.toolbar;
    }
    return toolbar;
};
var ɵ0$2 = getToolbar;
// prefer attributes than config
var computeOptions = function (options) {
    var computedOptions = {};
    var config = Object.assign({}, options.config, defaultConfig);
    computedOptions.toolbar = getToolbar(config);
    if (options.placeholder) {
        computedOptions.placeholder = options.placeholder;
    }
    return computedOptions;
};
var ɵ1 = computeOptions;

var NgxEditorComponent = /** @class */ (function () {
    function NgxEditorComponent() {
        this.placeholder = 'Type here...';
    }
    NgxEditorComponent_1 = NgxEditorComponent;
    NgxEditorComponent.prototype.writeValue = function (value) {
        if (!value) {
            return;
        }
        this.updateContent(value);
    };
    NgxEditorComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    NgxEditorComponent.prototype.registerOnTouched = function () { };
    NgxEditorComponent.prototype.parseDoc = function (contentJson) {
        return schema.nodeFromJSON(contentJson);
    };
    NgxEditorComponent.prototype.updateContent = function (value) {
        try {
            var doc = this.parseDoc(value);
            var state = this.view.state;
            var tr = state.tr;
            tr.replaceWith(0, state.doc.content.size, doc);
            this.view.dispatch(tr);
        }
        catch (err) {
            console.error('Unable to update document.', err);
        }
    };
    NgxEditorComponent.prototype.handleTransactions = function (tr) {
        var state = this.view.state.applyTransaction(tr).state;
        this.view.updateState(state);
        if (tr.docChanged) {
            var json = state.doc.toJSON();
            this.onChange(json);
        }
    };
    NgxEditorComponent.prototype.createEditor = function () {
        this.view = new EditorView(this.ngxEditor.nativeElement, {
            state: EditorState.create({
                schema: schema,
                plugins: getPlugins(this.options),
            }),
            dispatchTransaction: this.handleTransactions.bind(this),
            attributes: {
                class: 'NgxEditor-Content'
            },
        });
    };
    NgxEditorComponent.prototype.ngOnInit = function () {
        this.options = computeOptions({
            placeholder: this.placeholder,
            config: this.config
        });
        this.createEditor();
    };
    NgxEditorComponent.prototype.ngOnDestroy = function () {
        this.view.destroy();
    };
    var NgxEditorComponent_1;
    __decorate([
        ViewChild('ngxEditor', { static: true })
    ], NgxEditorComponent.prototype, "ngxEditor", void 0);
    __decorate([
        Input()
    ], NgxEditorComponent.prototype, "placeholder", void 0);
    __decorate([
        Input()
    ], NgxEditorComponent.prototype, "config", void 0);
    NgxEditorComponent = NgxEditorComponent_1 = __decorate([
        Component({
            selector: 'ngx-editor',
            template: "<div class=\"NgxEditor-Wrapper\">\n  <div #ngxEditor></div>\n</div>\n",
            providers: [{
                    provide: NG_VALUE_ACCESSOR,
                    useExisting: forwardRef(function () { return NgxEditorComponent_1; }),
                    multi: true
                }],
            encapsulation: ViewEncapsulation.None,
            styles: [".ProseMirror{outline:0}.NgxEditor-Wrapper{background:padding-box #fff;color:#000;border-radius:4px;border:2px solid rgba(0,0,0,.2)}.NgxEditor-MenuBar{display:flex;padding:.2rem;border-bottom:1px solid #ddd}.NgxEditor-MenuBar .NgxEditor-MenuItem{height:30px;width:30px;display:flex;align-items:center;justify-content:center;transition:.3s ease-in-out;border-radius:4px}.NgxEditor-MenuBar .NgxEditor-MenuItem:hover{cursor:pointer;background-color:#f1f1f1}.NgxEditor-MenuBar .NgxEditor-MenuItem.NgxEditor-MenuItem__active svg{fill:#06c}.NgxEditor-MenuBar .NgxEditor-MenuItem__Seperator{border-left:1px solid #ccc;margin:0 5px}.NgxEditor-Content{padding:.5rem}.NgxEditor-Content p{margin:0 0 .7rem}.NgxEditor-Placeholder{color:#6c757d;opacity:1}"]
        })
    ], NgxEditorComponent);
    return NgxEditorComponent;
}());

var NgxEditorModule = /** @class */ (function () {
    function NgxEditorModule() {
    }
    NgxEditorModule = __decorate([
        NgModule({
            declarations: [NgxEditorComponent],
            exports: [NgxEditorComponent],
        })
    ], NgxEditorModule);
    return NgxEditorModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { NgxEditorModule, NgxEditorComponent as ɵa };
//# sourceMappingURL=ngx-editor.js.map
